import java.net.*;
import java.io.*;
import java.util.*;

class Server implements Runnable {

  private static final int DEFAULT_PORT = 8765;
  //private static final int DEFAULT_PORT = 8799;
  private ServerSocket mainSock = null;
  private static Vector conVec = new Vector(7, 2); // connection vector

  public int this_port = DEFAULT_PORT;

  Thread runner;


  /**
   * constructor
   */
  public Server() {
     try {
        mainSock = new ServerSocket(this_port);
     } catch(Exception e) { 
        System.err.println("Can't create system socket, system exits."); 
        System.exit(1);
     }  
  }


  /**
   * waits for a connection then spawns a ServerManager to deal with it
   */
  public void run() {

     // continuously listen the socket
     while (true) {
        try {
           Socket this_connection = mainSock.accept();
	
           ServerManager manager = new ServerManager(this_connection, this);
           manager.start();
           conVec.addElement(manager);

           //clean up the vector if needed
           for (int i=0; i<conVec.size(); i++)
              if (!((ServerManager)(conVec.elementAt(i))).isAlive())
                  conVec.removeElementAt(i);
        } catch(Exception e) {
           System.err.println("Exception: " + e);
        }
     }
  }


  /**
   * start thread
   */   
  public void start() {
    if (runner == null); {
    runner = new Thread(this);
    runner.start();
    }
  }

  /**
   * stop thread
   */
  public void stop() {
    if (runner != null) {
    runner.stop();
    runner = null;
    }
  }


  public static void main(String[] args) {
     Server server = new Server();
     server.start();
  }

}

/////////////////////////////////////////////////////////


class ServerManager extends Thread {
   private Socket sock;
   private DataOutputStream out;
   private DataInputStream in;
   private PrintStream ps;

   private Server parent;

   /**
    * constructor
    * @param s server socket
    */
   public ServerManager(Socket s, Server parent) {
      sock = s;
      this.parent = parent;
   }


   /**
    * read commands from the client and do work
    */
   public void run() {
      try {
	out = new DataOutputStream(sock.getOutputStream());
	in  = new DataInputStream
	  (new BufferedInputStream(sock.getInputStream()));

        String hostname = sock.getInetAddress().getHostName();
        System.out.println("Connection from: " + hostname);
 

        doServerWork(hostname);
        sock.close();                   // close the socket

      } catch (Exception e) {
	e.printStackTrace();
      }
      stop();
   }

   ////////////////////////////////////////////////////
   // real worker
   // You need to write your codes in this function.
   // You don't need to change other parts until you know
   // know fully about all the codes.
   ///////////////////////////////////////////////////////
   private synchronized void doServerWork(String hostname) {
     
     File fl;
     FileOutputStream fout;
     PrintStream pout;
     String lNameTxt ,st;

     try {
       while((st = in.readLine()) != null) {
	 System.out.println("Got from the client: " + st);
	 if(st.equals("nodeset.ini")) {
	   fout = new FileOutputStream("nodeset.ini");
	   pout = new PrintStream(fout);
	   //while((st = in.readLine()) != null) {
	   st = in.readLine();
	   while(!st.equals("end")) {	
	     pout.println(st);
	     pout.println(in.readLine());
	     pout.println("");
	     st = in.readLine();
	   }
	   fout.close();
	 }
	 else if(st.equals("linkset.ini")) {
	   fout = new FileOutputStream("linkset.ini");
	   pout = new PrintStream(fout);
	   st = in.readLine();	   
	   while(!st.equals("end")) {
	     pout.println(st);
	     pout.println(in.readLine());
	     pout.println("");
	     st = in.readLine();
	   } 
	   fout.close();
	 }
	 else if(st.equals("dataset.ini")) {
	   fout = new FileOutputStream("dataset.ini");
	   pout = new PrintStream(fout);
	   st = in.readLine();	   
	   while(!st.equals("end")) {	   
	     pout.println(st);
	     st = in.readLine();	     
	   }
	   fout.close();
	 }
	 else if(st.equals("dist.ini")) {
	   fout = new FileOutputStream("dist.ini");
	   pout = new PrintStream(fout);
	   st = in.readLine();	   
	   while(!st.equals("end")) {	   
	     pout.println(st);
	     st = in.readLine();	     
	   }
	   fout.close();
	 }
	 else if(st.equals("exec.scrpt")) {
	   fout = new FileOutputStream("exec.scrpt");
	   pout = new PrintStream(fout);
	   st = in.readLine();
	   while(!st.equals("end.exec.script")) {	
	     pout.println(st);
	     st = in.readLine();
	   }
	   fout.close();
	 }
	 else if(st.equals("run")) {
	   try {
	     Process p  = Runtime.getRuntime().exec("./petasim");
	     DataInputStream dis = new DataInputStream
	       (new BufferedInputStream(p.getInputStream()));
	     
	     String s = null;
	     s = dis.readLine();
	     //while ((s = dis.readLine()) != null) {
	       out.writeBytes(s);
	       out.writeByte('\n'); 
	       System.out.println(s);
	     // }
	   }
	   catch (Exception e){
	     System.out.println("the detailed msg:" + e);
	     String s = " Error !  Please Check your Inputs.";
	     out.writeBytes(s);
	     out.writeByte('\n'); 
	     System.out.println(s);
	   }

	   //out.writeBytes("done");
	   //out.writeByte('\n');
	 }
	 else if(st.equals("result")) {
	   DataInputStream din;
	   FileInputStream fin;
	   fin = new FileInputStream("results.out");
	   din = new DataInputStream(fin);
	   out.writeBytes(din.readLine());
	   out.writeByte('\n');
	   out.writeBytes("done result");
	   out.writeByte('\n');
	   din.close();
	 }
	 else if(st.equals("clearResult")) {
	   Process p3  = Runtime.getRuntime().exec
	     ("/usr/bin/cp ./results.out ./results.out11");
	   Runtime rt = Runtime.getRuntime();
	   Process p2 = rt.exec
	     ("/usr/bin/cp ./results.clean ./results.out");
	 }
	 else if(st.equals("example")) {
	   DataInputStream din;
	   FileInputStream fin;
	   String fname, sin;
	   fname = in.readLine();
	   while(!fname.equals("done")) {
	     //System.out.println("----------: " + fname);
	     //--out.writeBytes(fname);
	     //--out.writeByte('\n');
	     fin = new FileInputStream(fname);
	     din = new DataInputStream(fin);
	     sin = din.readLine();
	     while(sin != null) {
	       //System.out.println("+++++++++: " + sin);
	       out.writeBytes(sin);
	       out.writeByte('\n');
	       out.writeBytes(" <");
	       out.writeByte('\n');
	       sin = din.readLine();
	       //System.out.println("+++++222: " + sin);
	     }
	    out.writeBytes("?");
	    out.writeByte('\n');
	    out.writeBytes("|");
	    out.writeByte('\n');
	    fname = in.readLine();
	    //System.out.println("-------222: " + fname);
	   }
	   out.writeBytes("done");
	   out.writeByte('\n');

	 }

       }
     }
     catch (Exception e) {
       e.printStackTrace();
     }
   }
}


